public inbox for passt-dev@passt.top
 help / color / mirror / code / Atom feed
From: David Gibson <david@gibson.dropbear.id.au>
To: passt-dev@passt.top, Stefano Brivio <sbrivio@redhat.com>
Cc: David Gibson <david@gibson.dropbear.id.au>
Subject: [PATCH v9 17/20] tcp: Pass TCP header and payload separately to tcp_update_check_tcp[46]()
Date: Mon,  4 Nov 2024 19:40:00 +1100	[thread overview]
Message-ID: <20241104084004.3544294-18-david@gibson.dropbear.id.au> (raw)
In-Reply-To: <20241104084004.3544294-1-david@gibson.dropbear.id.au>

Currently these expects both the TCP header and payload in a single IOV,
and goes to some trouble to locate the checksum field within it.  In the
current caller we've already know where the TCP header is, so we might as
well just pass it in.  This will need to work a bit differently for
vhost-user, but that code already needs to locate the TCP header for other
reasons, so again we can just pass it in.

Signed-off-by: David Gibson <david@gibson.dropbear.id.au>
---
 tcp.c          | 104 ++++++++++++-------------------------------------
 tcp_internal.h |   8 ++--
 tcp_vu.c       |  10 +++--
 3 files changed, 34 insertions(+), 88 deletions(-)

diff --git a/tcp_internal.h b/tcp_internal.h
index 2989db7..934fe9e 100644
--- a/tcp_internal.h
+++ b/tcp_internal.h
@@ -177,12 +177,12 @@ void tcp_rst_do(const struct ctx *c, struct tcp_tap_conn *conn);
 
 struct tcp_info_linux;
 
-void tcp_update_check_tcp4(const struct iphdr *iph,
+void tcp_update_check_tcp4(const struct iphdr *iph, struct tcphdr *th,
 			   const struct iovec *iov, int iov_cnt,
-			   size_t l4offset);
-void tcp_update_check_tcp6(const struct ipv6hdr *ip6h,
+			   size_t doffset);
+void tcp_update_check_tcp6(const struct ipv6hdr *ip6h, struct tcphdr *th,
 			   const struct iovec *iov, int iov_cnt,
-			   size_t l4offset);
+			   size_t doffset);
 void tcp_fill_headers4(const struct tcp_tap_conn *conn,
 		       struct tap_hdr *taph, struct iphdr *iph,
 		       struct tcp_payload_t *bp, size_t dlen,
diff --git a/tcp_vu.c b/tcp_vu.c
index 7a446b6..5073d67 100644
--- a/tcp_vu.c
+++ b/tcp_vu.c
@@ -69,15 +69,17 @@ static void tcp_vu_update_check(const struct flowside *tapside,
 	char *base = iov[0].iov_base;
 
 	if (inany_v4(&tapside->oaddr)) {
+		struct tcphdr *th = vu_payloadv4(base);
 		const struct iphdr *iph = vu_ip(base);
 
-		tcp_update_check_tcp4(iph, iov, iov_used,
-				      (char *)vu_payloadv4(base) - base);
+		tcp_update_check_tcp4(iph, th, iov, iov_used,
+				      (char *)(th + 1) - base);
 	} else {
+		struct tcphdr *th = vu_payloadv6(base);
 		const struct ipv6hdr *ip6h = vu_ip(base);
 
-		tcp_update_check_tcp6(ip6h, iov, iov_used,
-				      (char *)vu_payloadv6(base) - base);
+		tcp_update_check_tcp6(ip6h, th, iov, iov_used,
+				      (char *)(th + 1) - base);
 	}
 }
 
diff --git a/tcp.c b/tcp.c
index 8f09a3c..cf395c8 100644
--- a/tcp.c
+++ b/tcp.c
@@ -755,104 +755,48 @@ static void tcp_sock_set_bufsize(const struct ctx *c, int s)
 /**
  * tcp_update_check_tcp4() - Calculate TCP checksum for IPv4
  * @iph:	IPv4 header
  * @iov:	Pointer to the array of IO vectors
  * @iov_cnt:	Length of the array
  * @l4offset:	IPv4 payload offset in the iovec array
  */
-void tcp_update_check_tcp4(const struct iphdr *iph,
+void tcp_update_check_tcp4(const struct iphdr *iph, struct tcphdr *th,
 			   const struct iovec *iov, int iov_cnt,
-			   size_t l4offset)
+			   size_t doffset)
 {
 	uint16_t l4len = ntohs(iph->tot_len) - sizeof(struct iphdr);
 	struct in_addr saddr = { .s_addr = iph->saddr };
 	struct in_addr daddr = { .s_addr = iph->daddr };
-	size_t check_ofs;
-	__sum16 *check;
-	int check_idx;
 	uint32_t sum;
-	char *ptr;
 
 	sum = proto_ipv4_header_psum(l4len, IPPROTO_TCP, saddr, daddr);
 
-	check_idx = iov_skip_bytes(iov, iov_cnt,
-				   l4offset + offsetof(struct tcphdr, check),
-				   &check_ofs);
-
-	if (check_idx >= iov_cnt) {
-		err("TCP4 buffer is too small, iov size %zd, check offset %zd",
-		    iov_size(iov, iov_cnt),
-		    l4offset + offsetof(struct tcphdr, check));
-		return;
-	}
-
-	if (check_ofs + sizeof(*check) > iov[check_idx].iov_len) {
-		err("TCP4 checksum field memory is not contiguous "
-		    "check_ofs %zd check_idx %d iov_len %zd",
-		    check_ofs, check_idx, iov[check_idx].iov_len);
-		return;
-	}
-
-	ptr = (char *)iov[check_idx].iov_base + check_ofs;
-	if ((uintptr_t)ptr & (__alignof__(*check) - 1)) {
-		err("TCP4 checksum field is not correctly aligned in memory");
-		return;
-	}
-
-	check = (__sum16 *)ptr;
-
-	*check = 0;
-	*check = csum_iov(iov, iov_cnt, l4offset, sum);
+	th->check = 0;
+	sum = csum_unfolded(th, sizeof(*th), sum);
+	th->check = csum_iov(iov, iov_cnt, doffset, sum);
 }
 
 /**
  * tcp_update_check_tcp6() - Calculate TCP checksum for IPv6
  * @ip6h:	IPv6 header
  * @iov:	Pointer to the array of IO vectors
  * @iov_cnt:	Length of the array
  * @l4offset:	IPv6 payload offset in the iovec array
  */
-void tcp_update_check_tcp6(const struct ipv6hdr *ip6h,
+void tcp_update_check_tcp6(const struct ipv6hdr *ip6h, struct tcphdr *th,
 			   const struct iovec *iov, int iov_cnt,
-			   size_t l4offset)
+			   size_t doffset)
 {
 	uint16_t l4len = ntohs(ip6h->payload_len);
-	size_t check_ofs;
-	__sum16 *check;
-	int check_idx;
 	uint32_t sum;
-	char *ptr;
 
 	sum = proto_ipv6_header_psum(l4len, IPPROTO_TCP, &ip6h->saddr,
 				     &ip6h->daddr);
 
-	check_idx = iov_skip_bytes(iov, iov_cnt,
-				   l4offset + offsetof(struct tcphdr, check),
-				   &check_ofs);
-
-	if (check_idx >= iov_cnt) {
-		err("TCP6 buffer is too small, iov size %zd, check offset %zd",
-		    iov_size(iov, iov_cnt),
-		    l4offset + offsetof(struct tcphdr, check));
-		return;
-	}
-
-	if (check_ofs + sizeof(*check) > iov[check_idx].iov_len) {
-		err("TCP6 checksum field memory is not contiguous "
-		    "check_ofs %zd check_idx %d iov_len %zd",
-		    check_ofs, check_idx, iov[check_idx].iov_len);
-		return;
-	}
-
-	ptr = (char *)iov[check_idx].iov_base + check_ofs;
-	if ((uintptr_t)ptr & (__alignof__(*check) - 1)) {
-		err("TCP6 checksum field is not correctly aligned in memory");
-		return;
-	}
-
-	check = (__sum16 *)ptr;
-
-	*check = 0;
-	*check = csum_iov(iov, iov_cnt, l4offset, sum);
+	th->check = 0;
+	sum = csum_unfolded(th, sizeof(*th), sum);
+	th->check = csum_iov(iov, iov_cnt, doffset, sum);
 }
 
 /**
@@ -1004,11 +948,11 @@ void tcp_fill_headers4(const struct tcp_tap_conn *conn,
 		bp->th.check = 0;
 	} else {
 		const struct iovec iov = {
-			.iov_base = bp,
-			.iov_len = ntohs(iph->tot_len) - sizeof(struct iphdr),
+			.iov_base = bp->data,
+			.iov_len = dlen,
 		};
 
-		tcp_update_check_tcp4(iph, &iov, 1, 0);
+		tcp_update_check_tcp4(iph, &bp->th, &iov, 1, 0);
 	}
 
 	tap_hdr_update(taph, l3len + sizeof(struct ethhdr));
@@ -1051,11 +995,11 @@ void tcp_fill_headers6(const struct tcp_tap_conn *conn,
 		bp->th.check = 0;
 	} else {
 		const struct iovec iov = {
-			.iov_base = bp,
-			.iov_len = ntohs(ip6h->payload_len)
+			.iov_base = bp->data,
+			.iov_len = dlen,
 		};
 
-		tcp_update_check_tcp6(ip6h, &iov, 1, 0);
+		tcp_update_check_tcp6(ip6h, &bp->th, &iov, 1, 0);
 	}
 
 	tap_hdr_update(taph, l4len + sizeof(*ip6h) + sizeof(struct ethhdr));
@@ -755,104 +755,48 @@ static void tcp_sock_set_bufsize(const struct ctx *c, int s)
  * tcp_update_check_tcp4() - Calculate TCP checksum for IPv4
  * @iph:	IPv4 header
- * @iov:	Pointer to the array of IO vectors
- * @iov_cnt:	Length of the array
- * @l4offset:	IPv4 payload offset in the iovec array
+ * @th:		TCP header (updated)
+ * @iov:	IO vector containing the TCP payload
+ * @iov_cnt:	Length of @iov
+ * @doffset:	TCP payload offset in @iov
  */
@@ -755,104 +755,48 @@ static void tcp_sock_set_bufsize(const struct ctx *c, int s)
  * tcp_update_check_tcp6() - Calculate TCP checksum for IPv6
  * @ip6h:	IPv6 header
- * @iov:	Pointer to the array of IO vectors
- * @iov_cnt:	Length of the array
- * @l4offset:	IPv6 payload offset in the iovec array
+ * @th:		TCP header (updated)
+ * @iov:	IO vector containing the TCP payload
+ * @iov_cnt:	Length of @iov
+ * @doffset:	TCP payload offset in @iov
  */
-- 
2.47.0


  parent reply	other threads:[~2024-11-04  8:40 UTC|newest]

Thread overview: 24+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2024-11-04  8:39 [PATCH v9 00/20] (RFC) Add vhost-user support to passt. (part 3) David Gibson
2024-11-04  8:39 ` [PATCH v9 01/20] packet: replace struct desc by struct iovec David Gibson
2024-11-04  8:39 ` [PATCH v9 02/20] vhost-user: introduce virtio API David Gibson
2024-11-04  8:39 ` [PATCH v9 03/20] vhost-user: introduce vhost-user API David Gibson
2024-11-04  8:39 ` [PATCH v9 04/20] udp: Prepare udp.c to be shared with vhost-user David Gibson
2024-11-04  8:39 ` [PATCH v9 05/20] tcp: Export headers functions David Gibson
2024-11-04  8:39 ` [PATCH v9 06/20] passt: rename tap_sock_init() to tap_backend_init() David Gibson
2024-11-04  8:39 ` [PATCH v9 07/20] vhost-user: add vhost-user David Gibson
2024-11-04  8:39 ` [PATCH v9 08/20] test: Add tests for passt in vhost-user mode David Gibson
2024-11-04  8:39 ` [PATCH v9 09/20] vhost-user: Fix some trivial errors in comments David Gibson
2024-11-04  8:39 ` [PATCH v9 10/20] isolation: Abort if mode is not set David Gibson
2024-11-04  8:39 ` [PATCH v9 11/20] vhost-user: Consistent sense when encoding IP version as boolean David Gibson
2024-11-04  8:39 ` [PATCH v9 12/20] test: Fix trivial errors in two_guests_vu tests David Gibson
2024-11-04  8:39 ` [PATCH v9 13/20] tcp: Use only netinet/tcp.h instead of linux/tcp.h David Gibson
2024-11-04  8:39 ` [PATCH v9 14/20] tcp_vu: Share more header construction between IPv4 and IPv6 paths David Gibson
2024-11-04  8:39 ` [PATCH v9 15/20] tcp: Move tcp_l2_buf_fill_headers() to tcp_buf.c David Gibson
2024-11-04  8:39 ` [PATCH v9 16/20] tcp: Adjust iov_len before filling headers David Gibson
2024-11-04  8:40 ` David Gibson [this message]
2024-11-04  8:40 ` [PATCH v9 18/20] tcp: Pass TCP header and payload separately to tcp_fill_headers[46]() David Gibson
2024-11-04  8:40 ` [PATCH v9 19/20] tcp: Merge tcp_update_check_tcp[46]() David Gibson
2024-11-04  8:40 ` [PATCH v9 20/20] tcp: Merge tcp_fill_headers[46]() with each other David Gibson
2024-11-04  9:18 ` [PATCH v9 00/20] (RFC) Add vhost-user support to passt. (part 3) Stefano Brivio
2024-11-04  9:59   ` David Gibson
2024-11-05  1:10 ` David Gibson

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20241104084004.3544294-18-david@gibson.dropbear.id.au \
    --to=david@gibson.dropbear.id.au \
    --cc=passt-dev@passt.top \
    --cc=sbrivio@redhat.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
Code repositories for project(s) associated with this public inbox

	https://passt.top/passt

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for IMAP folder(s).